#!/usr/bin/env node
var program = require('commander');
var emoji = require('node-emoji');
var B = require('br4nch');
var _ = require('underscore');
var C = require('c0mm1t');
var I = require('1n1t');
var m4g1c = require('m4g1c');
var async = require('async');
var colors = require('colors');
var E = require('3x3c');
var updateNotifier = require('update-notifier');
var pkg = require('./package.json');
var CR = require('cr34t3');
var inquirer = require('inquirer');
var S = require('./lib/git-heads');
var opn = require('opn');
var isGitUrl = require('is-git-url');
var notify = require('./lib/Notify');
const assets = require('./lib/Assets.js');
var globalModulesDir = require('global-modules');
const os = require('os');
updateNotifier({pkg}).notify();

program
.option('-m, --message <message>', 'Commit message')
.option('-b, --new_branch <branch>', 'Git push origin as a new branch')
.option('-p, --publish', 'After process publish repo as a npm package')
.option('-i, --init', 'Git init')
.option('-s, --status', 'Git status')
.option('-c, --create', 'Create github repository')
.option('--clone', 'Clone github repository')
.option('-u, --update', 'Self update')
.option('-e, --enable', 'Enable Auto Commit')
.option('-d, --disable', 'Disable Auto Commit')
.parse(process.argv);

function sleep(ms) {
  return new Promise(function(resolve, reject) {
    setTimeout(function () {
      resolve();
    }, ms * 1000);
  });
}

function log(message) {
  console.log(emoji.emojify(':zap:'), colors.underline.white('Running') ,colors.cyan(message));
}

if (program.enable) {
  console.log(colors.green('g3l auto committer enabled in', process.cwd(), 'successfully. Happy coding..', emoji.emojify(':relaxed:')));
  var pm2 = require('pm2');
  var globalModulesDir = require('global-modules');

  pm2.connect(function(err) {
    if (err) {
      console.log(err);
      process.exit(2);
    }

    pm2.start({
      name: process.cwd(),
      script    : `${globalModulesDir}/g3l/lib/AutoCommit.js`,
      exec_mode : 'fork',
      max_memory_restart : '100M'
    }, function(err, apps) {
      pm2.disconnect();
      if (err) console.log(err);
    });
  });
} else if (program.disable) {
  console.log(colors.white('g3l auto committer disabled in', process.cwd(), 'successfully. '));
  var pm2 = require('pm2');
  pm2.delete(process.cwd(), function(err) {})
  pm2.disconnect();
  sleep(2)
   .then((value) => {
     console.log('Bye bye..', emoji.emojify(':white_frowning_face:'));
     process.exit();
   })
}

var commands = [
  {
    name: "new_branch",
    command: "git checkout -b ${branch}",
    description: "Git checkout new branch easily",
    priority: 99,
    boolean: false,
    containRequiredParam: true,
    function: "branch",
    params: [
      {
        name: "branch",
      }
    ]
  },
  {
    name: "init",
    command: "git init",
    description: "Git checkout new branch easily",
    priority: 100,
    boolean: true,
    containRequiredParam: false,
    function: "init",
    params: []
  },
  {
    name: "message",
    command: "git add . && git commit -m ${message} && git push origin ${branch}",
    description: "Git commit easily",
    priority: 95,
    boolean: false,
    containRequiredParam: true,
    function: "message",
    params: [
      {
        name: "message",
      }
    ],
  },
  {
    name: "publish",
    command: "npm version patch && npm publish",
    description: "Npm publish easily",
    boolean: true,
    function: "publish",
    priority: 80,
    containRequiredParam: false,
    params: [],
  },
  {
    name: "status",
    command: "git status",
    description: "Git status",
    boolean: true,
    function: "status",
    priority: 90,
    containRequiredParam: false,
    params: [],
  },
  {
    name: "create",
    command: "git create new repo",
    description: "Create new github repository",
    boolean: true,
    function: "create",
    priority: 90,
    containRequiredParam: false,
    params: [],
  },
  {
    name: "clone",
    command: "Git clone existing repository",
    description: "Git clone existing repository",
    boolean: true,
    function: "clone",
    priority: 89,
    containRequiredParam: false,
    params: [],
  },
  {
    name: "update",
    command: "Self update",
    description: "Self update",
    boolean: true,
    function: "update",
    priority: 1,
    containRequiredParam: false,
    params: [],
  }
];
function run(array) {
  return new Promise(function(resolve, reject) {
     array.forEach(function(piece) {
       if (eval('program.'+piece.name)) {
         if (piece.containRequiredParam || piece.boolean || eval('program.' + piece.name).length > 2) { /* If is has contain required param? */
            if (piece.boolean) {
              eval(piece.function + '(' + JSON.stringify(piece) + `).then((value) => {console.log(colors.grey(value));}).catch((err) => {console.log(colors.red(err));})`);
            } else {
              piece.params.forEach(function(param) {
                eval(piece.function + '(' + JSON.stringify(piece) + ').then((value) => {console.log(colors.grey(value));}).catch((err) => {console.log(colors.red(err));})');
              });
            }
         } else {
           process.exit();
         }
       }
     });
     resolve('')
  });
}

/* Sort by priority */
commands = _.sortBy(commands, 'priority').reverse();

/* Run commands */
run(commands)
  .then((value) => {console.log(value);})
  .catch((err) => {console.log(err);})

function init(command) {return new Promise(function(resolve, reject) {I().then((value) => {resolve(value)}).catch((err) => {reject(err)});});}

function branch(command) {
  return new Promise(function(resolve, reject) {
    log('Checkout new branch');
    E(`git checkout -b ${program.new_branch}`)
     .then((value) => {
       notify({title: 'g3l', 'message': `New branch created ${program.new_branch.trim()}`, 'status':'resolve'})
       resolve(`New branch created: ${program.new_branch}`);
     })
     .catch((err) => {
       notify({title: 'g3l', 'message': `New branch doesn\'t created as: ${program.new_branch.trim()}`, 'status':'reject'})
       reject(err)
     });
  });
}

function message(command) {
  return new Promise(function(resolve, reject) {
    log('Git commit and push proccess');
    var obj = {
      message: program.message,
      new: program.init,
    }
    C(obj)
      .then(function(value) {
        notify({title: 'g3l', 'message': 'Git committed successfully', 'status':'resolve'})
        resolve('Git committed successfully.');
      })
      .catch(function(err) {
        notify({title: 'g3l', 'message': `Git doesn\'t committed successfully.`, 'status':'reject'})
        reject(err)
      });
  });
}

function publish(command) {
  return new Promise(function(resolve, reject) {
    log('Version patch and publish as npm package');
    var isNew = program.init ? '' : 'npm version patch &&';
    E(`${isNew} npm publish`)
     .then((value) => {
       notify({title: 'g3l', 'message': 'Npm publish successfully', 'status':'resolve'})
       resolve(value);
     })
     .catch((err) => {
       notify({title: 'g3l', 'message': `Npm doesn\'t published successfully.`, 'status':'reject'})
       reject(err);
     })
  });
}

function status(command) {
  return new Promise(function(resolve, reject) {
    notify({title: 'g3l', 'message': 'Git status fetched successfully', 'status':'resolve'})
    S();
    resolve('Git status fetched successfully');
  });
}

function update(command) {
  return new Promise(function(resolve, reject) {
    log('g3l update process started.')
    E('npm i -g g3l')
      .then((value) => {
        notify({title: 'g3l', 'message': 'g3l updated successfully.', 'status':'resolve'})
        resolve('g3l updated successfully.')
      })
      .catch((err) => {
        notify({title: 'g3l', 'message': `g3l doesn\'t updated.`, 'status':'reject'})
        reject('g3l doesn\'t updated.')
      })
  });
}

function create(command) {
  return new Promise(function(resolve, reject) {
     CR()
       .then((value) => {
         log('Your repository url: ', value);
         opn(value);
         notify({title: 'g3l', 'message': 'g3l created github repository successfully', 'status':'resolve'})
         resolve(emoji.emojify(`:sunglasses: Horarayy! You can init your repository easily with this command: g3l -i`));
         process.exit();
       }).catch((err) => {
         notify({title: 'g3l', 'message': `Repository doesn\'t created successfully.`, 'status':'reject'})
         reject(err);
       });
  });
}

function clone(command) {
  return new Promise(function(resolve, reject) {
      var questions = [
        {
          type: 'input',
          name: 'url',
          message: 'What\'s your git repository url?',
          validate: function (value) {
            if(value.indexOf('.git') > -1 && value.trim().length > 1) {
              return true;
            }
            return 'Please write correct git repository.'
          }
        },
        {
          type: 'input',
          name: 'name',
          message: 'Which name you prefer for cloned repository directory name? (Optional)'
        },
      ];
      inquirer.prompt(questions).then(function (answers) {
        if (answers.name.trim().length < 1) {
          answers.name = answers.url.split('/').pop(-1).split('.git')[0];
        }
        E(`git clone ${answers.url} ${answers.name} && cd ${answers.name}`)
            .then((value) => {
              notify({title: 'g3l', 'message': 'Clone', 'subtitle': `Git clone successfully`, 'status':'resolve'})
              resolve('Clone done!')
            })
            .catch((err) => {
              notify({title: 'g3l', 'message': `Repository doesn\'t cloned successfully.`, 'status':'reject'})
              reject(err)
            });
      });
  });
}
